/* mbed Microcontroller Library
 * Copyright (c) 2006-2013 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "mbed.h"
#include "ble/BLE.h"
#include "ButtonService.h"
#include <time.h>

const static char     DEVICE_NAME[] = "LUMBERJACK_NANO";
static const uint16_t uuid16_list[] = {ButtonService::BUTTON_SERVICE_UUID};

Serial pc(USBTX, USBRX);


static ButtonService *buttonServicePtr;
bool isThereAConnection = false;

void sleep(unsigned int mseconds)
{
    clock_t goal = mseconds + clock();
    while (goal > clock());
}

void disconnectionCallback(const Gap::DisconnectionCallbackParams_t *params)
{
    BLE::Instance().gap().startAdvertising();
    isThereAConnection = false;
}

void connectionCallback(const Gap::ConnectionCallbackParams_t *params)
{
    pc.printf("Connection recieved!\r\n");
    isThereAConnection = true;
}

/**
 * This function is called when the ble initialization process has failled
 */
void onBleInitError(BLE &ble, ble_error_t error)
{
    /* Initialization error handling should go here */
}

/**
 * Callback triggered when the ble initialization process has finished
 */
void bleInitComplete(BLE::InitializationCompleteCallbackContext *params)
{
    BLE&        ble   = params->ble;
    ble_error_t error = params->error;

    if (error != BLE_ERROR_NONE) {
        /* In case of error, forward the error handling to onBleInitError */
        onBleInitError(ble, error);
        return;
    }

    /* Ensure that it is the default instance of BLE */
    if(ble.getInstanceID() != BLE::DEFAULT_INSTANCE) {
        return;
    }

    ble.gap().onDisconnection(disconnectionCallback);
    ble.gap().onConnection(connectionCallback);

    /* Setup primary service */
    buttonServicePtr = new ButtonService(ble, false /* initial value for button pressed */);

    /* setup advertising */
    ble.gap().accumulateAdvertisingPayload(GapAdvertisingData::BREDR_NOT_SUPPORTED | GapAdvertisingData::LE_GENERAL_DISCOVERABLE);
    ble.gap().accumulateAdvertisingPayload(GapAdvertisingData::COMPLETE_LIST_16BIT_SERVICE_IDS, (uint8_t *)uuid16_list, sizeof(uuid16_list));
    ble.gap().accumulateAdvertisingPayload(GapAdvertisingData::COMPLETE_LOCAL_NAME, (uint8_t *)DEVICE_NAME, sizeof(DEVICE_NAME));
    ble.gap().setAdvertisingType(GapAdvertisingParams::ADV_CONNECTABLE_UNDIRECTED);
    ble.gap().setAdvertisingInterval(1000); /* 1000ms. */
    pc.printf("start advertising now \r\n");
    ble.gap().startAdvertising();
}

int main(void)
{
    pc.printf("entering main!! \r\n");
    //uint8_t counter = 65; //'A'
    uint8_t counter[16] = {65, 65, 65, 65, 65,
                           65, 65, 65, 65, 65,
                           65, 65, 65, 65, 65,
                           0};

    BLE &ble = BLE::Instance();
    ble.init(bleInitComplete);
    
    pc.printf("entering spin loop\r\n");
    /* SpinWait for initialization to complete. This is necessary because the
     * BLE object is used in the main loop below. */
    while (ble.hasInitialized()  == false) { /* spin loop */ }
    pc.printf("leaving spin loop\r\n");
    
    while (true) {
        pc.printf("loop!\r\n");
        if(isThereAConnection) {
            //pc.printf("sending Notification: %d\r\n", counter);
            pc.printf("sending Notification: %d\r\n", counter[0]);
            buttonServicePtr->updateButtonState(counter);
            //counter++;
            for( int i = 0; i < 16; i++ ) {
                counter[i] = counter[i]+1;
            }
        } else {
            //
        }
        //ble.waitForEvent();
        sleep(1000);
    }
}
